@@ -8,6 +8,7 @@ class NotebooksClient(TagsSupportMixin, BaseClient):
     def create(
             self,
             machine_type,
+            project_id=None,
             cluster_id=None,
             container=None,
             container_id=None,
@@ -19,12 +20,18 @@ class NotebooksClient(TagsSupportMixin, BaseClient):
             shutdown_timeout=None,
             is_preemptible=None,
             is_public=None,
+            workspace=None,
+            workspace_ref=None,
+            workspace_username=None,
+            workspace_password=None,
             tags=None,
+            environment=None,
     ):
         """Create new notebook
 
         :param str machine_type:
         :param int container_id:
+        :param str|int project_id:
         :param str cluster_id:
         :param str container:
         :param str name:
@@ -36,6 +43,11 @@ class NotebooksClient(TagsSupportMixin, BaseClient):
         :param bool is_preemptible:
         :param bool is_public:
         :param list[str] tags: List of tags
+        :param str workspace: Project git repository url
+        :param str workspace_ref: Git commit hash, branch name or tag
+        :param str workspace_username: Project git repository username
+        :param str workspace_password: Project git repository password
+        :param dict environment: key value collection of envs that are used in notebook
 
         :return: Notebook ID
         :rtype str:
@@ -43,6 +55,7 @@ class NotebooksClient(TagsSupportMixin, BaseClient):
 
         notebook = models.Notebook(
             container_id=container_id,
+            project_id=project_id,
             cluster_id=cluster_id,
             container=container,
             name=name,
@@ -54,6 +67,11 @@ class NotebooksClient(TagsSupportMixin, BaseClient):
             is_preemptible=is_preemptible,
             machine_type=machine_type,
             is_public=is_public,
+            environment=environment,
+            workspace=workspace,
+            workspace_username=workspace_username,
+            workspace_password=workspace_password,
+            workspace_ref=workspace_ref,
         )
 
         repository = self.build_repository(repositories.CreateNotebook)
