@@ -1,6 +1,12 @@
 import attr
 
 
+@attr.s
+class VolumeOptions(object):
+    kind = attr.ib(type=str, default=None)
+    size = attr.ib(type=str, default=None)
+
+
 @attr.s
 class Dataset(object):
     uri = attr.ib(type=str, default=None)
@@ -9,3 +15,4 @@ class Dataset(object):
     etag = attr.ib(type=str, default=None)
     version_id = attr.ib(type=str, default=None)
     name = attr.ib(type=str, default=None)
+    volume_options = attr.ib(type=VolumeOptions, factory=VolumeOptions)
