@@ -1,3 +1,4 @@
+import json
 import os
 import shutil
 import tempfile
@@ -6,6 +7,7 @@ import mock
 import pytest
 from click.testing import CliRunner
 
+from gradient.api_sdk import sdk_exceptions
 from gradient.api_sdk.clients.http_client import default_headers
 from gradient.cli import cli
 from tests import example_responses, MockResponse
@@ -18,6 +20,40 @@ EXPECTED_HEADERS_WITH_CHANGED_API_KEY = EXPECTED_HEADERS.copy()
 EXPECTED_HEADERS_WITH_CHANGED_API_KEY["X-API-Key"] = "some_key"
 
 
+@pytest.fixture
+def basic_options_metrics_stream_websocket_connection_iterator():
+    def generator(self):
+        yield """{"handle": "jstkd2lapucirs", "object_type": "mljob", "chart_name": "memoryUsage",
+               "pod_metrics": {"mljob-ecgrgm7ok8chv-0-worker": {"time_stamp": 1588155670, "value": "5881856"}}}"""
+
+        yield """{"handle": "jstkd2lapucirs", "object_type": "mljob", "chart_name": "cpuPercentage",
+               "pod_metrics": {"mljob-ecgrgm7ok8chv-0-worker": {"time_stamp": 1588155670, "value": "0"}}}"""
+
+        yield """{"handle": "jstkd2lapucirs", "object_type": "mljob", "chart_name": "memoryUsage",
+               "pod_metrics": {"mljob-ecgrgm7ok8chv-0-worker": {"time_stamp": 1588155700, "value": "5881857"}}}"""
+
+        raise sdk_exceptions.EndWebsocketStream("keton")
+
+    return generator
+
+
+@pytest.fixture
+def all_options_metrics_stream_websocket_connection_iterator():
+    def generator(self):
+        yield """{"handle": "jstkd2lapucirs", "object_type": "mljob", "chart_name": "gpuMemoryUsed",
+               "pod_metrics": {"mljob-ecgrgm7ok8chv-0-worker": {"time_stamp": 1588155670, "value": "5881856"}}}"""
+
+        yield """{"handle": "jstkd2lapucirs", "object_type": "mljob", "chart_name": "gpuMemoryFree",
+               "pod_metrics": {"mljob-ecgrgm7ok8chv-0-worker": {"time_stamp": 1588155670, "value": "0"}}}"""
+
+        yield """{"handle": "jstkd2lapucirs", "object_type": "mljob", "chart_name": "gpuMemoryUsed",
+               "pod_metrics": {"mljob-ecgrgm7ok8chv-0-worker": {"time_stamp": 1588155700, "value": "5881857"}}}"""
+
+        raise sdk_exceptions.EndWebsocketStream("keton")
+
+    return generator
+
+
 class TestJobs(object):
     EXPECTED_STDOUT_WITH_WRONG_API_TOKEN = "Failed to fetch data: Invalid API token\n"
     RESPONSE_JSON_WITH_WRONG_API_TOKEN = {"status": 400, "message": "Invalid API token"}
@@ -195,12 +231,6 @@ class TestJobLogs(TestJobs):
     COMMAND_WITH_OPTIONS_FILE = ["jobs", "logs", "--optionsFile", ]  # path added in test
     REQUEST_WITH_OPTIONS_FILE = {"jobId": "some_id", "line": 50, "limit": 200}
 
-    EXPECTED_STDOUT_WITHOUT_PARAMETERS = """Usage: cli jobs logs [OPTIONS]
-Try "cli jobs logs --help" for help.
-
-Error: Missing option '--id'.
-"""
-
     EXPECTED_STDOUT = """+Job some_id logs------------------------------------------------------------------------+
 | LINE | MESSAGE                                                                         |
 +------+---------------------------------------------------------------------------------+
@@ -220,15 +250,6 @@ Error: Missing option '--id'.
 
     EXPECTED_STDOUT_WITH_WRONG_API_TOKEN = "Failed to fetch data: Invalid API token\n"
 
-    @mock.patch("gradient.api_sdk.clients.http_client.requests.get")
-    def test_command_should_not_send_request_without_required_parameters(self, get_patched):
-        cli_runner = CliRunner()
-        result = cli_runner.invoke(cli.cli, self.COMMAND_WITHOUT_REQUIRED_PARAMETERS)
-
-        get_patched.assert_not_called()
-        assert result.exit_code == 2
-        assert result.output == self.EXPECTED_STDOUT_WITHOUT_PARAMETERS
-
     @mock.patch("gradient.api_sdk.clients.http_client.requests.get")
     def test_should_send_valid_get_request_and_print_available_logs(self, get_patched):
         get_patched.return_value = MockResponse(json_data=self.EXPECTED_RESPONSE_JSON)
@@ -373,14 +394,14 @@ class TestListJobArtifacts(TestJobs):
 
     @mock.patch("gradient.api_sdk.clients.http_client.requests.get")
     def test_should_send_valid_get_request_with_all_parameters_for_a_list_of_artifacts(self, get_patched):
-        get_patched.return_value = MockResponse()
+        get_patched.return_value = MockResponse(LIST_JOB_FILES_RESPONSE_JSON)
         job_id = "some_job_id"
         result = self.runner.invoke(cli.cli,
                                     ["jobs", "artifacts", "list", "--id", job_id, "--apiKey", "some_key", "--size",
                                      "--links",
                                      "--files", "foo"])
 
-        get_patched.assert_called_with("{}/jobs/artifactsList".format(self.URL),
+        get_patched.assert_called_with("{}/jobs/artifactsListV2".format(self.URL),
                                        headers=EXPECTED_HEADERS_WITH_CHANGED_API_KEY,
                                        json=None,
                                        params={"jobId": job_id,
@@ -391,11 +412,11 @@ class TestListJobArtifacts(TestJobs):
 
     @mock.patch("gradient.api_sdk.clients.http_client.requests.get")
     def test_should_read_options_from_yaml_file(self, get_patched, jobs_artifacts_list_config_path):
-        get_patched.return_value = MockResponse()
+        get_patched.return_value = MockResponse(LIST_JOB_FILES_RESPONSE_JSON)
         command = ["jobs", "artifacts", "list", "--optionsFile", jobs_artifacts_list_config_path]
         result = self.runner.invoke(cli.cli, command)
 
-        get_patched.assert_called_with("{}/jobs/artifactsList".format(self.URL),
+        get_patched.assert_called_with("{}/jobs/artifactsListV2".format(self.URL),
                                        headers=EXPECTED_HEADERS_WITH_CHANGED_API_KEY,
                                        json=None,
                                        params={"files": "keton*.py",
@@ -413,12 +434,12 @@ class TestListJobArtifacts(TestJobs):
                                                                                                               get_patched,
                                                                                                               option,
                                                                                                               param):
-        get_patched.return_value = MockResponse(status_code=200)
+        get_patched.return_value = MockResponse(LIST_JOB_FILES_RESPONSE_JSON, status_code=200)
         job_id = "some_job_id"
         result = self.runner.invoke(cli.cli,
                                     ["jobs", "artifacts", "list", "--id", job_id, "--apiKey", "some_key"] + [option])
 
-        get_patched.assert_called_with("{}/jobs/artifactsList".format(self.URL),
+        get_patched.assert_called_with("{}/jobs/artifactsListV2".format(self.URL),
                                        headers=EXPECTED_HEADERS_WITH_CHANGED_API_KEY,
                                        json=None,
                                        params={"jobId": job_id,
@@ -527,7 +548,7 @@ class TestJobsCreate(object):
     RESPONSE_CONTENT_200 = b'{"handle":"sadkfhlskdjh","message":"success"}\n'
     EXPECTED_STDOUT = u'New job created with ID: sadkfhlskdjh\n'
     EXPECTED_STDOUT_TAGS = u'New job created with ID: sadkfhlskdjh\n' \
-                           u'https://www.paperspace.com/console/jobs/sadkfhlskdjh\n'
+                           u'https://console.paperspace.com/jobs/sadkfhlskdjh\n'
 
     RESPONSE_JSON_404_PROJECT_NOT_FOUND = {"details": {"handle": "wrong_handle"}, "error": "Project not found"}
     RESPONSE_CONTENT_404_PROJECT_NOT_FOUND = b'{"details":{"handle":"wrong_handle"},"error":"Project not found"}\n'
@@ -614,7 +635,7 @@ class TestJobsCreate(object):
 
 class TestDownloadJobArtifacts(TestJobs):
     runner = CliRunner()
-    LIST_FILES_URL = "https://api.paperspace.io/jobs/artifactsList"
+    LIST_FILES_URL = "https://api.paperspace.io/jobs/artifactsListV2"
     DESTINATION_DIR_NAME = "dest"
     DESTINATION_DIR_PATH = os.path.join(tempfile.gettempdir(), "dest")
 
@@ -685,3 +706,505 @@ class TestDownloadJobArtifacts(TestJobs):
             assert h.read() == "\"Elo\n\""
 
         assert result.exit_code == 0
+
+
+class TestJobsMetricsGetCommand(object):
+    GET_JOB_URL = "https://api.paperspace.io/jobs/getPublicJob"
+    GET_METRICS_URL = "https://aws-testing.paperspace.io/metrics/api/v1/range"
+    BASIC_OPTIONS_COMMAND = [
+        "jobs", "metrics", "get",
+        "--id", "jstkd2lapucirs",
+    ]
+    ALL_OPTIONS_COMMAND = [
+        "jobs", "metrics", "get",
+        "--id", "jstkd2lapucirs",
+        "--metric", "gpuMemoryFree",
+        "--metric", "gpuMemoryUsed",
+        "--interval", "20s",
+        "--start", "2020-04-01",
+        "--end", "2020-04-02 21:37:00",
+        "--apiKey", "some_key",
+    ]
+    FULL_OPTIONS_COMMAND_WITH_OPTIONS_FILE = [
+        "jobs", "metrics", "get",
+        "--optionsFile",  # path added in test,
+    ]
+
+    GET_JOB_REQUEST_JSON = {"jobId": "jstkd2lapucirs"}
+    BASIC_COMMAND_GET_METRICS_REQUEST_PARAMS = {
+        "start": "2020-04-29T10:11:07Z",
+        "handle": "jstkd2lapucirs",
+        "interval": "30s",
+        "charts": "cpuPercentage,memoryUsage",
+        "objecttype": "mljob",
+    }
+    ALL_COMMANDS_GET_METRICS_REQUEST_PARAMS = {
+        "start": "2020-04-01T00:00:00Z",
+        "handle": "jstkd2lapucirs",
+        "interval": "20s",
+        "charts": "gpuMemoryFree,gpuMemoryUsed",
+        "objecttype": "mljob",
+        "end": "2020-04-02T21:37:00Z",
+    }
+
+    GET_JOB_RESPONSE_JSON = example_responses.GET_JOB_RESPONSE
+    GET_METRICS_RESPONSE_JSON = example_responses.GET_JOB_METRICS_RESPONSE
+    GET_JOB_RESPONSE_JSON_WHEN_NO_JOBS_WERE_FOUND = {
+        "error": {
+            "name": "ApplicationError",
+            "status": 404,
+            "message": "No such job",
+        },
+    }
+
+    EXPECTED_STDOUT = """{
+  "cpuPercentage": {
+    "mljob-ecgrgm7ok8chv-0-worker": [
+      {
+        "time_stamp": 1588155157, 
+        "value": "0"
+      }, 
+      {
+        "time_stamp": 1588155187, 
+        "value": "0"
+      }, 
+      {
+        "time_stamp": 1588155217, 
+        "value": "0"
+      }
+    ]
+  }, 
+  "memoryUsage": {
+    "mljob-ecgrgm7ok8chv-0-worker": [
+      {
+        "time_stamp": 1588155097, 
+        "value": "0"
+      }, 
+      {
+        "time_stamp": 1588155127, 
+        "value": "5881856"
+      }, 
+      {
+        "time_stamp": 1588155157, 
+        "value": "5881856"
+      }, 
+      {
+        "time_stamp": 1588155187, 
+        "value": "5881856"
+      }
+    ]
+  }
+}
+"""
+
+    EXPECTED_STDOUT_WHEN_INVALID_API_KEY_WAS_USED = "Failed to fetch data: Invalid API token\n"
+    EXPECTED_STDOUT_WHEN_EXPERIMENT_WAS_NOT_FOUND = "Failed to fetch data: No such job\n"
+    EXPECTED_STDOUT_WHEN_NO_METRICS_FOUND = """{
+    "cpuPercentage": null,
+    "memoryUsage": null
+}
+"""
+    EXPECTED_STDOUT_WHEN_ERROR_CODE_WAS_RETURNED_WITHOUT_ERROR_MESSAGE = "Failed to fetch data\n"
+
+    @mock.patch("gradient.api_sdk.clients.http_client.requests.get")
+    @mock.patch("gradient.api_sdk.clients.http_client.requests.post")
+    def test_should_read_all_available_metrics_when_metrics_get_command_was_used_with_basic_options(
+            self, post_patched, get_patched):
+        post_patched.return_value = MockResponse(self.GET_JOB_RESPONSE_JSON)
+        get_patched.return_value = MockResponse(self.GET_METRICS_RESPONSE_JSON)
+
+        runner = CliRunner()
+        result = runner.invoke(cli.cli, self.BASIC_OPTIONS_COMMAND)
+
+        assert json.loads(result.output.strip()) == json.loads(self.EXPECTED_STDOUT.strip()), \
+            str(result.output) + str(result.exc_info)
+        post_patched.assert_called_once_with(
+            self.GET_JOB_URL,
+            json=self.GET_JOB_REQUEST_JSON,
+            params=None,
+            data=None,
+            files=None,
+            headers=EXPECTED_HEADERS,
+        )
+        get_patched.assert_called_once_with(
+            self.GET_METRICS_URL,
+            json=None,
+            params=self.BASIC_COMMAND_GET_METRICS_REQUEST_PARAMS,
+            headers=EXPECTED_HEADERS,
+        )
+        assert result.exit_code == 0, result.exc_info
+
+    @mock.patch("gradient.api_sdk.clients.http_client.requests.get")
+    @mock.patch("gradient.api_sdk.clients.http_client.requests.post")
+    def test_should_read_metrics_when_metrics_get_command_was_used_with_all_options(
+            self, post_patched, get_patched):
+        post_patched.return_value = MockResponse(self.GET_JOB_RESPONSE_JSON)
+        get_patched.return_value = MockResponse(self.GET_METRICS_RESPONSE_JSON)
+
+        runner = CliRunner()
+        result = runner.invoke(cli.cli, self.ALL_OPTIONS_COMMAND)
+
+        # comparing objects instead of strings because Py2 and Py3 produce slightly different outputs
+        assert json.loads(result.output.strip()) == json.loads(self.EXPECTED_STDOUT.strip()), result.exc_info
+        post_patched.assert_called_once_with(
+            self.GET_JOB_URL,
+            json=self.GET_JOB_REQUEST_JSON,
+            params=None,
+            data=None,
+            files=None,
+            headers=EXPECTED_HEADERS_WITH_CHANGED_API_KEY,
+        )
+        get_patched.assert_called_once_with(
+            self.GET_METRICS_URL,
+            json=None,
+            params=self.ALL_COMMANDS_GET_METRICS_REQUEST_PARAMS,
+            headers=EXPECTED_HEADERS_WITH_CHANGED_API_KEY,
+        )
+
+        assert result.exit_code == 0, result.exc_info
+
+    @mock.patch("gradient.api_sdk.clients.http_client.requests.get")
+    @mock.patch("gradient.api_sdk.clients.http_client.requests.post")
+    def test_should_read_metrics_when_metrics_get_was_executed_and_options_file_was_used(
+            self, post_patched, get_patched, jobs_metrics_get_config_path):
+        post_patched.return_value = MockResponse(self.GET_JOB_RESPONSE_JSON)
+        get_patched.return_value = MockResponse(self.GET_METRICS_RESPONSE_JSON)
+
+        command = self.FULL_OPTIONS_COMMAND_WITH_OPTIONS_FILE[:] + [jobs_metrics_get_config_path]
+        runner = CliRunner()
+        result = runner.invoke(cli.cli, command)
+
+        # comparing objects instead of strings because Py2 and Py3 produce slightly different outputs
+        assert json.loads(result.output.strip()) == json.loads(self.EXPECTED_STDOUT.strip()), result.exc_info
+        post_patched.assert_called_once_with(
+            self.GET_JOB_URL,
+            json=self.GET_JOB_REQUEST_JSON,
+            params=None,
+            data=None,
+            files=None,
+            headers=EXPECTED_HEADERS_WITH_CHANGED_API_KEY,
+        )
+        get_patched.assert_called_once_with(
+            self.GET_METRICS_URL,
+            json=None,
+            params=self.ALL_COMMANDS_GET_METRICS_REQUEST_PARAMS,
+            headers=EXPECTED_HEADERS_WITH_CHANGED_API_KEY,
+        )
+        assert result.exit_code == 0, result.exc_info
+
+    @mock.patch("gradient.api_sdk.clients.http_client.requests.post")
+    def test_should_print_valid_error_message_when_invalid_api_key_was_used(self, post_patched):
+        post_patched.return_value = MockResponse({"status": 400, "message": "Invalid API token"}, status_code=400)
+
+        runner = CliRunner()
+        result = runner.invoke(cli.cli, self.ALL_OPTIONS_COMMAND)
+
+        assert result.output == self.EXPECTED_STDOUT_WHEN_INVALID_API_KEY_WAS_USED, result.exc_info
+
+        post_patched.assert_called_once_with(
+            self.GET_JOB_URL,
+            json=self.GET_JOB_REQUEST_JSON,
+            params=None,
+            data=None,
+            files=None,
+            headers=EXPECTED_HEADERS_WITH_CHANGED_API_KEY,
+        )
+
+        assert result.exit_code == 0, result.exc_info
+
+    @mock.patch("gradient.api_sdk.clients.http_client.requests.post")
+    def test_should_print_valid_error_message_when_job_was_not_found(self, post_patched):
+        post_patched.return_value = MockResponse(self.GET_JOB_RESPONSE_JSON_WHEN_NO_JOBS_WERE_FOUND, status_code=400)
+
+        runner = CliRunner()
+        result = runner.invoke(cli.cli, self.ALL_OPTIONS_COMMAND)
+
+        assert result.output == self.EXPECTED_STDOUT_WHEN_EXPERIMENT_WAS_NOT_FOUND, result.exc_info
+
+        post_patched.assert_called_once_with(
+            self.GET_JOB_URL,
+            json=self.GET_JOB_REQUEST_JSON,
+            params=None,
+            data=None,
+            files=None,
+            headers=EXPECTED_HEADERS_WITH_CHANGED_API_KEY,
+        )
+        assert result.exit_code == 0, result.exc_info
+
+    @mock.patch("gradient.api_sdk.clients.http_client.requests.get")
+    @mock.patch("gradient.api_sdk.clients.http_client.requests.post")
+    def test_should_print_valid_message_when_was_no_metrics_were_returned(
+            self, post_patched, get_patched):
+        post_patched.return_value = MockResponse(self.GET_JOB_RESPONSE_JSON)
+        get_patched.return_value = MockResponse(example_responses.JOBS_METRICS_GET_RESPONSE_WHEN_NO_DATA_WAS_FOUND)
+
+        runner = CliRunner()
+        result = runner.invoke(cli.cli, self.ALL_OPTIONS_COMMAND)
+
+        assert json.loads(result.output.strip()) == json.loads(self.EXPECTED_STDOUT_WHEN_NO_METRICS_FOUND.strip()), \
+            result.exc_info
+
+        post_patched.assert_called_once_with(
+            self.GET_JOB_URL,
+            json=self.GET_JOB_REQUEST_JSON,
+            params=None,
+            data=None,
+            files=None,
+            headers=EXPECTED_HEADERS_WITH_CHANGED_API_KEY,
+        )
+        get_patched.assert_called_once_with(
+            self.GET_METRICS_URL,
+            json=None,
+            params=self.ALL_COMMANDS_GET_METRICS_REQUEST_PARAMS,
+            headers=EXPECTED_HEADERS_WITH_CHANGED_API_KEY,
+        )
+
+        assert result.exit_code == 0, result.exc_info
+
+    @mock.patch("gradient.api_sdk.clients.http_client.requests.get")
+    @mock.patch("gradient.api_sdk.clients.http_client.requests.post")
+    def test_should_print_valid_error_message_when_error_code_was_returned_without_error_message(
+            self, post_patched, get_patched):
+        post_patched.return_value = MockResponse(self.GET_JOB_RESPONSE_JSON)
+        get_patched.return_value = MockResponse(status_code=500)
+
+        runner = CliRunner()
+        result = runner.invoke(cli.cli, self.ALL_OPTIONS_COMMAND)
+
+        assert result.output == self.EXPECTED_STDOUT_WHEN_ERROR_CODE_WAS_RETURNED_WITHOUT_ERROR_MESSAGE, result.exc_info
+        post_patched.assert_called_once_with(
+            self.GET_JOB_URL,
+            json=self.GET_JOB_REQUEST_JSON,
+            params=None,
+            data=None,
+            files=None,
+            headers=EXPECTED_HEADERS_WITH_CHANGED_API_KEY,
+        )
+        get_patched.assert_called_once_with(
+            self.GET_METRICS_URL,
+            json=None,
+            params=self.ALL_COMMANDS_GET_METRICS_REQUEST_PARAMS,
+            headers=EXPECTED_HEADERS_WITH_CHANGED_API_KEY,
+        )
+
+        assert result.exit_code == 0, result.exc_info
+
+
+class TestJobsMetricsStreamCommand(object):
+    GET_JOB_URL = "https://api.paperspace.io/jobs/getPublicJob"
+    GET_METRICS_URL = "https://aws-testing.paperspace.io/metrics/api/v1/stream"
+    BASIC_OPTIONS_COMMAND = [
+        "jobs", "metrics", "stream",
+        "--id", "jstkd2lapucirs",
+    ]
+    ALL_OPTIONS_COMMAND = [
+        "jobs", "metrics", "stream",
+        "--id", "jstkd2lapucirs",
+        "--metric", "gpuMemoryFree",
+        "--metric", "gpuMemoryUsed",
+        "--interval", "20s",
+        "--apiKey", "some_key",
+    ]
+    ALL_OPTIONS_COMMAND_WITH_OPTIONS_FILE = [
+        "jobs", "metrics", "stream",
+        "--optionsFile",  # path added in test,
+    ]
+
+    GET_JOB_REQUEST_JSON = {"jobId": "jstkd2lapucirs"}
+    BASIC_COMMAND_CHART_DESCRIPTOR = '{"chart_names": ["cpuPercentage", "memoryUsage"], "handles": ["jstkd2lapucirs"]' \
+                                     ', "object_type": "mljob", "poll_interval": "30s"}'
+    ALL_COMMANDS_CHART_DESCRIPTOR = '{"chart_names": ["gpuMemoryFree", "gpuMemoryUsed"], "handles": ["jstkd2lapucirs"' \
+                                    '], "object_type": "mljob", "poll_interval": "20s"}'
+
+    GET_JOB_RESPONSE_JSON = example_responses.GET_JOB_RESPONSE
+
+    EXPECTED_TABLE_1 = """+------------------------------+---------------+-------------+
+| Pod                          | cpuPercentage | memoryUsage |
++------------------------------+---------------+-------------+
+| mljob-ecgrgm7ok8chv-0-worker |               | 5881856     |
++------------------------------+---------------+-------------+"""
+    EXPECTED_TABLE_2 = """+------------------------------+---------------+-------------+
+| Pod                          | cpuPercentage | memoryUsage |
++------------------------------+---------------+-------------+
+| mljob-ecgrgm7ok8chv-0-worker | 0             | 5881856     |
++------------------------------+---------------+-------------+"""
+    EXPECTED_TABLE_3 = """+------------------------------+---------------+-------------+
+| Pod                          | cpuPercentage | memoryUsage |
++------------------------------+---------------+-------------+
+| mljob-ecgrgm7ok8chv-0-worker | 0             | 5881857     |
++------------------------------+---------------+-------------+"""
+
+    ALL_OPTIONS_EXPECTED_TABLE_1 = """+------------------------------+---------------+---------------+
+| Pod                          | gpuMemoryFree | gpuMemoryUsed |
++------------------------------+---------------+---------------+
+| mljob-ecgrgm7ok8chv-0-worker |               | 5881856       |
++------------------------------+---------------+---------------+"""
+    ALL_OPTIONS_EXPECTED_TABLE_2 = """+------------------------------+---------------+---------------+
+| Pod                          | gpuMemoryFree | gpuMemoryUsed |
++------------------------------+---------------+---------------+
+| mljob-ecgrgm7ok8chv-0-worker | 0             | 5881856       |
++------------------------------+---------------+---------------+"""
+    ALL_OPTIONS_EXPECTED_TABLE_3 = """+------------------------------+---------------+---------------+
+| Pod                          | gpuMemoryFree | gpuMemoryUsed |
++------------------------------+---------------+---------------+
+| mljob-ecgrgm7ok8chv-0-worker | 0             | 5881857       |
++------------------------------+---------------+---------------+"""
+
+    EXPECTED_STDOUT_WHEN_INVALID_API_KEY_WAS_USED = "Failed to fetch data: Incorrect API Key provided\nForbidden\n"
+    EXPECTED_STDOUT_WHEN_JOB_WAS_NOT_FOUND = "Failed to fetch data: No such job\n"
+
+    @mock.patch("gradient.commands.common.TerminalPrinter")
+    @mock.patch("gradient.api_sdk.repositories.common.websocket.create_connection")
+    @mock.patch("gradient.api_sdk.clients.http_client.requests.post")
+    def test_should_read_all_available_metrics_when_metrics_get_command_was_used_with_basic_options(
+            self, post_patched, create_ws_connection_patched, terminal_printer_cls_patched,
+            basic_options_metrics_stream_websocket_connection_iterator):
+        post_patched.return_value = MockResponse(self.GET_JOB_RESPONSE_JSON)
+
+        ws_connection_instance_mock = mock.MagicMock()
+        ws_connection_instance_mock.__iter__ = basic_options_metrics_stream_websocket_connection_iterator
+        create_ws_connection_patched.return_value = ws_connection_instance_mock
+
+        runner = CliRunner()
+        result = runner.invoke(cli.cli, self.BASIC_OPTIONS_COMMAND)
+
+        terminal_printer_cls_patched().init.assert_called_once()
+        terminal_printer_cls_patched().rewrite_screen.assert_has_calls([
+            mock.call(self.EXPECTED_TABLE_1),
+            mock.call(self.EXPECTED_TABLE_2),
+            mock.call(self.EXPECTED_TABLE_3),
+        ])
+        terminal_printer_cls_patched().cleanup.assert_called_once()
+
+        post_patched.assert_called_once_with(
+            self.GET_JOB_URL,
+            json=self.GET_JOB_REQUEST_JSON,
+            params=None,
+            data=None,
+            files=None,
+            headers=EXPECTED_HEADERS,
+        )
+        ws_connection_instance_mock.send.assert_called_once_with(self.BASIC_COMMAND_CHART_DESCRIPTOR)
+        assert result.exit_code == 0, result.exc_info
+
+    @mock.patch("gradient.commands.common.TerminalPrinter")
+    @mock.patch("gradient.api_sdk.repositories.common.websocket.create_connection")
+    @mock.patch("gradient.api_sdk.clients.http_client.requests.post")
+    def test_should_read_metrics_when_metrics_get_command_was_used_with_all_options(
+            self, post_patched, create_ws_connection_patched, terminal_printer_cls_patched,
+            all_options_metrics_stream_websocket_connection_iterator):
+        post_patched.return_value = MockResponse(self.GET_JOB_RESPONSE_JSON)
+
+        ws_connection_instance_mock = mock.MagicMock()
+        ws_connection_instance_mock.__iter__ = all_options_metrics_stream_websocket_connection_iterator
+        create_ws_connection_patched.return_value = ws_connection_instance_mock
+
+        runner = CliRunner()
+        result = runner.invoke(cli.cli, self.ALL_OPTIONS_COMMAND)
+
+        terminal_printer_cls_patched().init.assert_called_once()
+        terminal_printer_cls_patched().rewrite_screen.assert_has_calls([
+            mock.call(self.ALL_OPTIONS_EXPECTED_TABLE_1),
+            mock.call(self.ALL_OPTIONS_EXPECTED_TABLE_2),
+            mock.call(self.ALL_OPTIONS_EXPECTED_TABLE_3),
+        ])
+        terminal_printer_cls_patched().cleanup.assert_called_once()
+
+        post_patched.assert_called_once_with(
+            self.GET_JOB_URL,
+            json=self.GET_JOB_REQUEST_JSON,
+            params=None,
+            data=None,
+            files=None,
+            headers=EXPECTED_HEADERS_WITH_CHANGED_API_KEY,
+        )
+
+        ws_connection_instance_mock.send.assert_called_once_with(self.ALL_COMMANDS_CHART_DESCRIPTOR)
+        assert result.exit_code == 0, result.exc_info
+
+    @mock.patch("gradient.commands.common.TerminalPrinter")
+    @mock.patch("gradient.api_sdk.repositories.common.websocket.create_connection")
+    @mock.patch("gradient.api_sdk.clients.http_client.requests.post")
+    def test_should_read_metrics_when_metrics_get_was_executed_and_options_file_was_used(
+            self, post_patched, create_ws_connection_patched, terminal_printer_cls_patched,
+            all_options_metrics_stream_websocket_connection_iterator,
+            jobs_metrics_stream_config_path):
+        post_patched.return_value = MockResponse(self.GET_JOB_RESPONSE_JSON)
+        ws_connection_instance_mock = mock.MagicMock()
+        ws_connection_instance_mock.__iter__ = all_options_metrics_stream_websocket_connection_iterator
+        create_ws_connection_patched.return_value = ws_connection_instance_mock
+
+        command = self.ALL_OPTIONS_COMMAND_WITH_OPTIONS_FILE[:] + [jobs_metrics_stream_config_path]
+        runner = CliRunner()
+        result = runner.invoke(cli.cli, command)
+
+        terminal_printer_cls_patched().init.assert_called_once()
+        terminal_printer_cls_patched().rewrite_screen.assert_has_calls([
+            mock.call(self.ALL_OPTIONS_EXPECTED_TABLE_1),
+            mock.call(self.ALL_OPTIONS_EXPECTED_TABLE_2),
+            mock.call(self.ALL_OPTIONS_EXPECTED_TABLE_3),
+        ])
+        terminal_printer_cls_patched().cleanup.assert_called_once()
+
+        post_patched.assert_called_once_with(
+            self.GET_JOB_URL,
+            json=self.GET_JOB_REQUEST_JSON,
+            params=None,
+            data=None,
+            files=None,
+            headers=EXPECTED_HEADERS_WITH_CHANGED_API_KEY,
+        )
+
+        ws_connection_instance_mock.send.assert_called_once_with(self.ALL_COMMANDS_CHART_DESCRIPTOR)
+        assert result.exit_code == 0, result.exc_info
+
+    @mock.patch("gradient.commands.common.TerminalPrinter")
+    @mock.patch("gradient.api_sdk.repositories.common.websocket.create_connection")
+    @mock.patch("gradient.api_sdk.clients.http_client.requests.post")
+    def test_should_print_valid_error_message_when_invalid_api_key_was_used(
+            self, post_patched, create_ws_connection_patched, terminal_printer_cls_patched):
+        post_patched.return_value = MockResponse({"status": 400, "message": "Invalid API token"}, 400)
+
+        runner = CliRunner()
+        result = runner.invoke(cli.cli, self.ALL_OPTIONS_COMMAND)
+
+        assert "Failed to fetch data: Invalid API token\n" == result.output, result.exc_info
+
+        post_patched.assert_called_once_with(
+            self.GET_JOB_URL,
+            json=self.GET_JOB_REQUEST_JSON,
+            params=None,
+            data=None,
+            files=None,
+            headers=EXPECTED_HEADERS_WITH_CHANGED_API_KEY,
+        )
+
+        create_ws_connection_patched.assert_not_called()
+        assert result.exit_code == 0, result.exc_info
+
+    @mock.patch("gradient.commands.common.TerminalPrinter")
+    @mock.patch("gradient.api_sdk.repositories.common.websocket.create_connection")
+    @mock.patch("gradient.api_sdk.clients.http_client.requests.post")
+    def test_should_print_valid_error_message_when_job_was_not_found(
+            self, post_patched, create_ws_connection_patched, terminal_printer_cls_patched):
+        post_patched.return_value = MockResponse(
+            {"error": {"name": "ApplicationError", "status": 404, "message": "No such job"}},
+            404,
+        )
+
+        runner = CliRunner()
+        result = runner.invoke(cli.cli, self.ALL_OPTIONS_COMMAND)
+
+        assert result.output == self.EXPECTED_STDOUT_WHEN_JOB_WAS_NOT_FOUND, result.exc_info
+
+        post_patched.assert_called_once_with(
+            self.GET_JOB_URL,
+            json=self.GET_JOB_REQUEST_JSON,
+            params=None,
+            data=None,
+            files=None,
+            headers=EXPECTED_HEADERS_WITH_CHANGED_API_KEY,
+        )
+
+        create_ws_connection_patched.assert_not_called()
+        assert result.exit_code == 0, result.exc_info
