@@ -53,14 +53,17 @@ class TestNotebooksCreate(object):
         "create",
         "--machineType", "P5000",
         "--container", "jupyter/notebook",
-        "--clusterId", "321"
+        "--projectId", "pr1234",
+        "--clusterId", "321",
     ]
     EXPECTED_REQUEST_JSON = {
         "machineType": "P5000",
         "container": "jupyter/notebook",
         "clusterId": "321",
-        'isPreemptible': False,
-        'isPublic': False,
+        "isPreemptible": False,
+        "isPublic": False,
+        "workspace": "none",
+        "projectId": "pr1234",
     }
     EXPECTED_RESPONSE_JSON = {
         "handle": "some_id",
@@ -71,13 +74,14 @@ class TestNotebooksCreate(object):
         "container": "jupyter/notebook",
     }
     EXPECTED_STDOUT = "Created new notebook with id: some_id\n" \
-                      "https://www.paperspace.com/some_namespace/notebook/prg284tu2\n"
+                      "https://console.paperspace.com/some_namespace/notebook/prg284tu2\n"
 
     COMMAND_WITH_API_KEY_USED = [
         "notebooks",
         "create",
         "--machineType", "P5000",
         "--container", "jupyter/notebook",
+        "--projectId", "pr1234",
         "--clusterId", "321",
         "--apiKey", "some_key",
     ]
@@ -87,6 +91,7 @@ class TestNotebooksCreate(object):
         "create",
         "--machineType", "P5000",
         "--container", "jupyter/notebook",
+        "--projectId", "pr1234",
         "--clusterId", "321",
         "--name", "some_notebook_name",
         "--registryUsername", "some_username",
@@ -94,20 +99,25 @@ class TestNotebooksCreate(object):
         "--command", "some_entrypoint",
         "--containerUser", "some_container_user",
         "--shutdownTimeout", "8",
+        "--environment", '{"key":"val"}',
         "--isPreemptible",
+        "--workspace", "https://github.com/fake/repo.git",
     ]
     EXPECTED_REQUEST_JSON_WITH_ALL_OPTIONS = {
         "machineType": "P5000",
         "container": "jupyter/notebook",
+        "projectId": "pr1234",
         "clusterId": "321",
         "name": "some_notebook_name",
         "registryUsername": "some_username",
         "registryPassword": "some_password",
         "command": "c29tZV9lbnRyeXBvaW50",
         "containerUser": "some_container_user",
+        "environment": {"key": "val"},
         "shutdownTimeout": 8,
         "isPreemptible": True,
         "isPublic": False,
+        "workspace": "https://github.com/fake/repo.git",
     }
     COMMAND_WITH_OPTIONS_FILE_USED = ["notebooks", "create", "--optionsFile", ]  # path added in test
 
@@ -344,7 +354,7 @@ class TestNotebooksStart(object):
     RESPONSE_JSON_WITH_WRONG_API_TOKEN = {"status": 400, "message": "Invalid API token"}
     EXPECTED_STDOUT_WITH_WRONG_API_TOKEN = "Failed to create resource: Invalid API token\n"
     EXPECTED_STDOUT_WITH_KEY = "Started notebook with id: n123\n" \
-                               "https://www.paperspace.com/some_namespace/notebook/prg284tu2\n"
+                               "https://console.paperspace.com/some_namespace/notebook/prg284tu2\n"
 
     @mock.patch("gradient.api_sdk.clients.http_client.requests.get")
     @mock.patch("gradient.api_sdk.clients.http_client.requests.post")
